package com.sxtvs.open.job;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.BulkRequest;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import com.sxtvs.open.api.news.entity.YoumeiData;
import com.sxtvs.open.api.news.entity.YoumeiOffset;
import com.sxtvs.open.api.news.service.impl.YoumeiDataServiceImpl;
import com.sxtvs.open.api.news.service.impl.YoumeiOffsetServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Component
@Slf4j
public class PutEsJob {
    @Resource
    private ElasticsearchClient elasticsearchClient;

    @Resource
    private YoumeiDataServiceImpl youmeiDataService;

    @Resource
    private YoumeiOffsetServiceImpl youmeiOffsetService;

    @Scheduled(cron = "0 21 * * * *")
    public void putData()  {
        log.info("ES 入数据开始");
        YoumeiOffset youmeiOffset = youmeiOffsetService.getById(4);
        long offset = youmeiOffset.getOffset() + 1L;
        long max = 0L;
        while (true){
            List<YoumeiData> youmeiDataList = youmeiDataService.lambdaQuery().gt(YoumeiData::getOffset, offset).last("limit 1000").list();
            log.info("本次数据{}条，offset从{}开始",youmeiDataList.size(), offset);
            if (youmeiDataList.size() == 0){
                break;
            }
            List<BulkOperation> bulkOperations = new ArrayList<>();

            youmeiDataList.forEach(a -> bulkOperations.add(BulkOperation.of(b -> b.index(c -> c.id(String.valueOf(a.getOffset())).document(a)))));

            try {
                elasticsearchClient.bulk(BulkRequest.of(x ->x.index("news_data").operations(bulkOperations)));
            } catch (IOException e) {
                e.printStackTrace();
            }
            max = youmeiDataList.get(youmeiDataList.size()-1).getOffset();
            offset += 1000;
        }
        youmeiOffset.setOffset(max);
        youmeiOffsetService.updateById(youmeiOffset);
        log.info("ES 入数据结束");
    }
}
